@@ -1,10 +1,11 @@
 import argparse
 import logging
+from typing import List
 
 from . import dispatch_open
 
 
-def main(args):
+def main(args_list: List[str]) -> None:
 
     from .. import config_dir, config_file
 
@@ -36,7 +37,7 @@ def main(args):
         help="open the configuration directory in your native file browser",
     )
 
-    args = parser.parse_args(args)
+    args = parser.parse_args(args_list)
 
     if args.list:
         print(config_dir)
