@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import logging
 import os
 import signal
@@ -13,16 +15,14 @@ from typing import (
     Dict,
     Iterable,
     Iterator,
-    List,
     Optional,
     TextIO,
-    Tuple,
     Union,
     cast,
 )
 
 import pyModeS as pms
-from tqdm.autonotebook import tqdm
+from tqdm.rich import tqdm
 
 import pandas as pd
 
@@ -100,7 +100,7 @@ def decode_time_dump1090(
     return now
 
 
-decode_time: Dict[str, Callable[[str, Optional[datetime]], datetime]] = {
+decode_time: dict[str, Callable[[str, Optional[datetime]], datetime]] = {
     "radarcape": decode_time_radarcape,
     "dump1090": decode_time_dump1090,
     "default": decode_time_default,
@@ -187,7 +187,7 @@ class Aircraft(object):
         self.icao24 = icao24
         self._callsign: Optional[str] = None
         self._flight: Optional[Flight] = None
-        self.cumul: List[Entry] = []
+        self.cumul: list[Entry] = []
 
         self.t0: Optional[datetime] = None
         self.t1: Optional[datetime] = None
@@ -245,7 +245,7 @@ class Aircraft(object):
         return self._callsign
 
     @callsign.setter
-    def callsign(self, args: Tuple[datetime, str]) -> None:
+    def callsign(self, args: tuple[datetime, str]) -> None:
         t, msg = args
         callsign = pms.adsb.callsign(msg).strip("_")
         if callsign == "":
@@ -261,7 +261,7 @@ class Aircraft(object):
         pass
 
     @speed.setter
-    def speed(self, args: Tuple[datetime, str]) -> None:
+    def speed(self, args: tuple[datetime, str]) -> None:
         t, msg = args
         vdata = pms.adsb.velocity(msg)
         if vdata is None:
@@ -297,7 +297,7 @@ class Aircraft(object):
         pass
 
     @position.setter
-    def position(self, args: Tuple[datetime, str]) -> None:
+    def position(self, args: tuple[datetime, str]) -> None:
         t, msg = args
         oe = pms.adsb.oe_flag(msg)
         setattr(self, "m" + str(oe), msg)
@@ -336,7 +336,7 @@ class Aircraft(object):
         pass
 
     @surface.setter
-    def surface(self, args: Tuple[datetime, str]) -> None:
+    def surface(self, args: tuple[datetime, str]) -> None:
         t, msg = args
         self.lat, self.lon = pms.adsb.surface_position_with_ref(
             msg, self.lat0, self.lon0
@@ -362,7 +362,7 @@ class Aircraft(object):
         pass
 
     @altcode.setter
-    def altcode(self, args: Tuple[datetime, str]) -> None:
+    def altcode(self, args: tuple[datetime, str]) -> None:
         t, msg = args
         from pyModeS import hex2bin
 
@@ -379,7 +379,7 @@ class Aircraft(object):
         pass
 
     @idcode.setter
-    def idcode(self, args: Tuple[datetime, str]) -> None:
+    def idcode(self, args: tuple[datetime, str]) -> None:
         t, msg = args
         from pyModeS import hex2bin
 
@@ -400,7 +400,7 @@ class Aircraft(object):
         pass
 
     @bds20.setter
-    def bds20(self, args: Tuple[datetime, str]) -> None:
+    def bds20(self, args: tuple[datetime, str]) -> None:
         t, msg = args
         callsign = pms.commb.cs20(msg).strip("_")
         if callsign == "":
@@ -426,7 +426,7 @@ class Aircraft(object):
         pass
 
     @bds40.setter
-    def bds40(self, args: Tuple[datetime, str]) -> None:
+    def bds40(self, args: tuple[datetime, str]) -> None:
         t, msg = args
         with self.lock:
             # in case altitude was already included from altcode (DF 4 or 20)
@@ -464,7 +464,7 @@ class Aircraft(object):
         pass
 
     @bds44.setter
-    def bds44(self, args: Tuple[datetime, str]) -> None:
+    def bds44(self, args: tuple[datetime, str]) -> None:
         t, msg = args
         wind = pms.commb.wind44(msg)
         wind = wind if wind is not None else (None, None)
@@ -512,7 +512,7 @@ class Aircraft(object):
         pass
 
     @bds45.setter
-    def bds45(self, args: Tuple[datetime, str]) -> None:
+    def bds45(self, args: tuple[datetime, str]) -> None:
         t, msg = args
         with self.lock:
             # in case altitude was already included from altcode (DF 4 or 20)
@@ -570,7 +570,7 @@ class Aircraft(object):
         pass
 
     @bds50.setter
-    def bds50(self, args: Tuple[datetime, str]) -> None:
+    def bds50(self, args: tuple[datetime, str]) -> None:
         t, msg = args
         with self.lock:
             # in case altitude was already included from altcode (DF 4 or 20)
@@ -617,7 +617,7 @@ class Aircraft(object):
         pass
 
     @bds60.setter
-    def bds60(self, args: Tuple[datetime, str]) -> None:
+    def bds60(self, args: tuple[datetime, str]) -> None:
         t, msg = args
         with self.lock:
             # in case altitude was already included from altcode (DF 4 or 20)
@@ -663,7 +663,7 @@ class Aircraft(object):
         pass
 
     @nuc_p.setter
-    def nuc_p(self, args: Tuple[datetime, str]) -> None:
+    def nuc_p(self, args: tuple[datetime, str]) -> None:
         t, msg = args
         with self.lock:
             hpl, rcu, rcv = pms.adsb.nuc_p(msg)
@@ -690,7 +690,7 @@ class Aircraft(object):
         pass
 
     @nic_v1.setter
-    def nic_v1(self, args: Tuple[datetime, str]) -> None:
+    def nic_v1(self, args: tuple[datetime, str]) -> None:
         t, msg = args
         if self.nic_s is None:
             return
@@ -717,7 +717,7 @@ class Aircraft(object):
         pass
 
     @nic_v2.setter
-    def nic_v2(self, args: Tuple[datetime, str]) -> None:
+    def nic_v2(self, args: tuple[datetime, str]) -> None:
         t, msg = args
         if self.nic_a is None or self.nic_bc is None:
             return
@@ -743,7 +743,7 @@ class Aircraft(object):
         pass
 
     @nuc_r.setter
-    def nuc_r(self, args: Tuple[datetime, str]) -> None:
+    def nuc_r(self, args: tuple[datetime, str]) -> None:
         t, msg = args
         with self.lock:
             hve, vve = pms.adsb.nuc_v(msg)
@@ -768,7 +768,7 @@ class Aircraft(object):
         pass
 
     @nac_v.setter
-    def nac_v(self, args: Tuple[datetime, str]) -> None:
+    def nac_v(self, args: tuple[datetime, str]) -> None:
         t, msg = args
         with self.lock:
             hfm, vfm = pms.adsb.nac_v(msg)
@@ -793,7 +793,7 @@ class Aircraft(object):
         pass
 
     @nac_p.setter
-    def nac_p(self, args: Tuple[datetime, str]) -> None:
+    def nac_p(self, args: tuple[datetime, str]) -> None:
         t, msg = args
         with self.lock:
             epu, vepu = pms.adsb.nac_p(msg)
@@ -818,7 +818,7 @@ class Aircraft(object):
         pass
 
     @sil.setter
-    def sil(self, args: Tuple[datetime, str]) -> None:
+    def sil(self, args: tuple[datetime, str]) -> None:
         t, msg = args
         with self.lock:
             phcr, pvpl, base = pms.adsb.sil(msg, self.version)
@@ -858,12 +858,79 @@ class AircraftDict(Dict[str, Aircraft]):
             ac.lon0 = lon0
 
 
-class Decoder:
+class DumpFormat:
+    def __init__(self, template: str, sep: str = ",") -> None:
+
+        self.template = template
+        self.sep = sep
+        self.cols = list(x.strip() for x in template.split(sep))
+        time_gen = (i for i, elt in enumerate(self.cols) if elt == "time")
+        self.time_index = next(time_gen, None)
+        if self.time_index is None:
+            msg = "Format invalid: must contain 'time'"
+            raise ValueError(msg)
+
+        long_gen = (i for i, elt in enumerate(self.cols) if elt == "longmsg")
+        self.msg_index = next(long_gen, None)
+        self.splitmsg = slice(18, None)
+
+        if self.msg_index is not None:
+            return
+
+        short_gen = (i for i, elt in enumerate(self.cols) if elt == "shortmsg")
+        self.msg_index = next(short_gen, None)
+        if self.msg_index is None:
+            msg = "Format invalid: must contain either 'longmsg' or 'shortmsg'"
+            raise ValueError(msg)
+
+        self.splitmsg = slice(None)
+
+    def get_timestamp(self, line: str) -> datetime:
+        elts = line.split(self.sep)
+        return datetime.fromtimestamp(
+            float(elts[self.time_index].strip()), timezone.utc  # type: ignore
+        )
+
+    def get_msg(self, line: str) -> str:
+        elts = line.split(self.sep)
+        return elts[self.msg_index][self.splitmsg].strip()  # type: ignore
+
+
+class ModeS_Decoder:
+    """
+
+    This class implements the full data decoding process with the `pyModeS
+    library <https://mode-s.org/api/>`_
+
+    Data is first available as a list of messages per aircraft, but can be
+    exported as a :class:`~traffic.core.Traffic` object.
+
+    .. warning::
+
+        The :meth:`from_address`, :meth:`from_dump1090`, and :meth:`from_rtlsdr`
+        classmethods start a decoding thread on the creation of the object.  The
+        thread can be stopped with a ``decoder.thread.stop()`` call.
+
+    :param reference: A reference location must be provided to decode ground
+        messages. A reference can be set as:
+
+        - a tuple of latitude/longitude coordinates
+        - a str calling for an :class:`~traffic.core.structure.Airport`
+          structure
+
+        If no reference is provided (None), then reference location (latitude=0,
+        longitude=0) is considered until more data can be used to readjust the
+        reference.
+
+    """
+
     thread: Optional[StoppableThread]
 
     def __init__(
-        self, reference: Union[None, str, Airport, Tuple[float, float]] = None
+        self,
+        reference: None | str | Airport | tuple[float, float] = None,
     ) -> None:
+        """ """
         if isinstance(reference, str):
             from ...data import airports
 
@@ -886,24 +953,52 @@ class Decoder:
     @classmethod
     def from_file(
         cls,
-        filename: Union[str, Path],
-        reference: Union[str, Airport, Tuple[float, float]],
+        filename: str | Path,
+        reference: str | Airport | tuple[float, float],
         uncertainty: bool = False,
-    ) -> "Decoder":
+        template: str = "time, longmsg",
+        sep: str = ",",
+    ) -> "ModeS_Decoder":
+        """Decode raw messages dumped in a text file.
+
+        The file should contain for each line at least a timestamp and an
+        hexadecimal message, as a CSV-like format.
+
+        :param filename: the path to the file containing the data
+
+        :param reference: the reference location, as specified above
+
+        :param uncertainty: if True, decode also `uncertainty information
+            <https://mode-s.org/decode/content/ads-b/7-uncertainty.html>`_
+
+        :param template: the header explaining how data is organised
+
+            Three parameters are accepted:
+
+            - ``time`` represents the timestamp in seconds (float)
+            - ``shortmsg`` represents the regular version of the ADS-B
+              hexadecimal message (messages of length 28 for ADS-B)
+            - ``longmsg`` represents messages containing timestamp information
+              as a prefix, as dumped by many decoding softwares, such as
+              `dump1090 <https://github.com/MalcolmRobb/dump1090/>`_ or other
+              receivers.
+
+            By default, the expected format is ``time, longmsg``
+        """
 
         if isinstance(filename, str):
             filename = Path(filename)
 
+        dumpformat = DumpFormat(template, sep)
+
         with filename.open("r") as fh:
             all_lines = fh.readlines()
             decoder = cls(reference)
             decoder.process_msgs(
                 list(
                     (
-                        datetime.fromtimestamp(
-                            float(line.strip().split(",")[0]), timezone.utc
-                        ),
-                        line.strip().split(",")[1][18:],
+                        dumpformat.get_timestamp(line),
+                        dumpformat.get_msg(line),
                     )
                     for line in all_lines
                 ),
@@ -915,17 +1010,17 @@ class Decoder:
     def from_binary(
         cls,
         filename: Union[str, Path],
-        reference: Union[str, Airport, Tuple[float, float]],
+        reference: Union[str, Airport, tuple[float, float]],
         *,
         uncertainty: bool = False,
         time_fmt: str = "dump1090",
         time_0: Optional[datetime] = None,
         redefine_mag: int = 10,
         fh: Optional[TextIO] = None,
-    ) -> "Decoder":
+    ) -> "ModeS_Decoder":
 
         decoder = cls(reference)
-        redefine_freq = 2 ** redefine_mag - 1
+        redefine_freq = 2**redefine_mag - 1
         decode_time_here = decode_time.get(time_fmt, decode_time_default)
 
         def next_in_binary(filename: Union[str, Path]) -> Iterator[bytes]:
@@ -936,6 +1031,9 @@ class Decoder:
                         return
                     yield get
 
+        # We don't know the size of the binary so tqdm.rich does not work
+        from tqdm.autonotebook import tqdm
+
         for i, bin_msg in tqdm(enumerate(next_msg(next_in_binary(filename)))):
 
             if len(bin_msg) < 23:
@@ -958,10 +1056,30 @@ class Decoder:
     @classmethod
     def from_rtlsdr(
         cls,
-        reference: Union[str, Airport, Tuple[float, float]],
-        file_pattern: str = "~/ADSB_EHS_RAW_%Y%m%d_dump1090.csv",
+        reference: Union[str, Airport, tuple[float, float]],
+        file_pattern: str = "~/ADSB_EHS_RAW_%Y%m%d_rtlsdr.csv",
         uncertainty: bool = False,
-    ) -> "Decoder":  # coverage: ignore
+    ) -> "ModeS_Decoder":  # coverage: ignore
+        """Decode raw messages dumped from a RTL-SDR receiver.
+
+        :param reference: the reference location, as specified above
+        :param file_pattern: the filename where to dump received hexadecimal
+            messages
+
+            Timestamp format specifiers are accepted.
+
+            | Default value: ``"~/ADSB_EHS_RAW_%Y%m%d_rtlsdr.csv"``
+            | (The ``~`` character gets expanded as your home directory)
+
+        :param uncertainty: if True, decode also `uncertainty information
+            <https://mode-s.org/decode/content/ads-b/7-uncertainty.html>`_
+
+        .. warning::
+
+            This method requires the `pyrtlsdr
+            <https://github.com/roger-/pyrtlsdr>`_ optional dependency.
+
+        """
 
         from .rtlsdr import MyRtlReader
 
@@ -983,17 +1101,17 @@ class Decoder:
     def from_socket(
         cls,
         socket: socket.socket,
-        reference: Union[str, Airport, Tuple[float, float]],
+        reference: Union[str, Airport, tuple[float, float]],
         *,
         uncertainty: bool,
         time_fmt: str = "default",
         time_0: Optional[datetime] = None,
         redefine_mag: int = 7,
         fh: Optional[TextIO] = None,
-    ) -> "Decoder":  # coverage: ignore
+    ) -> "ModeS_Decoder":  # coverage: ignore
 
         decoder = cls(reference)
-        redefine_freq = 2 ** redefine_mag - 1
+        redefine_freq = 2**redefine_mag - 1
         decode_time_here = decode_time.get(time_fmt, decode_time_default)
 
         def next_in_socket() -> Iterator[bytes]:
@@ -1040,10 +1158,30 @@ class Decoder:
     @classmethod
     def from_dump1090(
         cls,
-        reference: Union[str, Airport, Tuple[float, float]],
+        reference: Union[str, Airport, tuple[float, float]],
         file_pattern: str = "~/ADSB_EHS_RAW_%Y%m%d_dump1090.csv",
         uncertainty: bool = False,
-    ) -> "Decoder":  # coverage: ignore
+    ) -> "ModeS_Decoder":  # coverage: ignore
+        """Decode raw messages dumped from `dump1090
+        <https://github.com/MalcolmRobb/dump1090/>`_
+
+        :param reference: the reference location, as specified above
+        :param file_pattern: the filename where to dump received hexadecimal
+            messages
+
+            Timestamp format specifiers are accepted.
+
+            | Default value: ``"~/ADSB_EHS_RAW_%Y%m%d_dump1090.csv"``
+            | (The ``~`` character gets expanded as your home directory)
+
+        :param uncertainty: if True, decode also `uncertainty information
+            <https://mode-s.org/decode/content/ads-b/7-uncertainty.html>`_
+
+        .. warning::
+
+            dump1090 must be run the ``--net`` option.
+
+        """
         now = datetime.now(timezone.utc)
         filename = now.strftime(file_pattern)
         today = os.path.expanduser(filename)
@@ -1064,10 +1202,42 @@ class Decoder:
         cls,
         host: str,
         port: int,
-        reference: Union[str, Airport, Tuple[float, float]],
+        reference: Union[str, Airport, tuple[float, float]],
         file_pattern: str = "~/ADSB_EHS_RAW_%Y%m%d_tcp.csv",
+        time_fmt: str = "radarcape",
         uncertainty: bool = False,
-    ) -> "Decoder":  # coverage: ignore
+    ) -> "ModeS_Decoder":  # coverage: ignore
+        """Decode raw messages transmitted over a TCP network.
+
+        The file should contain for each line at least a timestamp and an
+        hexadecimal message, as a CSV-like format.
+
+        :param host: the IP address of the host to connect
+        :param port: the port of the host to connect
+        :param reference: the reference location, as specified above
+        :param file_pattern: the filename where to dump received hexadecimal
+            messages
+
+            Timestamp format specifiers are accepted.
+
+            | Default value: ``"~/ADSB_EHS_RAW_%Y%m%d_tcp.csv"``
+            | (The ``~`` character gets expanded as your home directory)
+
+        :param time_fmt: (default: ``"radarcape"``)
+
+            - if set to ``"radarcape"``, timestamp information included in the
+              long message is assumed to be GPS adjusted and is therefore used
+              to adjust the timestamp of each message;
+            - if set to ``"dump1090"``, clock information provided by the
+              software is used to adjust the timestamp of each message, but the
+              first timestamp is based on the computer clock;
+            - if set to ``"default"``, the timestamp from the computer is used
+              for all decoded messages
+
+        :param uncertainty: if True, decode also `uncertainty information
+            <https://mode-s.org/decode/content/ads-b/7-uncertainty.html>`_
+
+        """
         now = datetime.now(timezone.utc)
         filename = now.strftime(file_pattern)
         today = os.path.expanduser(filename)
@@ -1075,7 +1245,7 @@ class Decoder:
         s.connect((host, port))
         fh = open(today, "a", 1)
         return cls.from_socket(
-            s, reference, uncertainty=uncertainty, time_fmt="radarcape", fh=fh
+            s, reference, uncertainty=uncertainty, time_fmt=time_fmt, fh=fh
         )
 
     def redefine_reference(self, time: datetime) -> None:
@@ -1094,7 +1264,7 @@ class Decoder:
             self.acs.set_latlon(sum_lat / n, sum_lon / n)
 
     def process_msgs(
-        self, msgs: Iterable[Tuple[datetime, str]], uncertainty: bool = False
+        self, msgs: Iterable[tuple[datetime, str]], uncertainty: bool = False
     ) -> None:
 
         for i, (time, msg) in tqdm(enumerate(msgs), total=sum(1 for _ in msgs)):
@@ -1243,7 +1413,7 @@ class Decoder:
                 return
 
     @property
-    def aircraft(self) -> List[Dict[str, Any]]:
+    def aircraft(self) -> list[Dict[str, Any]]:
         return sorted(
             (
                 dict(
@@ -1265,7 +1435,12 @@ class Decoder:
         )
 
     @property
-    def traffic(self) -> Optional[Traffic]:
+    def traffic(self) -> None | Traffic:
+        """
+        :return: All decoded data is converted into a
+            :class:`~traffic.core.Traffic` object.
+
+        """
         try:
             return Traffic.from_flights(
                 self[elt["icao24"]] for elt in self.aircraft
