@@ -10,7 +10,7 @@ from traffic.core import Airspace
 from ...core import Flight, Traffic
 
 _current_dir = Path(__file__).parent
-__all__ = list(f.stem[:-5] for f in _current_dir.glob("**/*.json.gz"))
+__all__ = sorted(f.stem[:-5] for f in _current_dir.glob("**/*.json.gz"))
 
 
 @lru_cache()
